@@ -1,7 +1,6 @@
 // ***************************************************************** -*- C++ -*-
-
 // geotag.cpp, $Rev: 2286 $
-// Sample program to read gpx files and update the images with GPS tags
+// Sample program to read gpx files and update images with GPS tags
 
 #include <exiv2/exiv2.hpp>
 
@@ -17,7 +16,7 @@
 #include <sys/stat.h>
 #include <sys/types.h>
 
-#include "expat.h"
+#include <expat.h>
 
 #include <vector>
 #include <string>
@@ -28,14 +27,6 @@
 # endif
 #endif
 
-#ifdef __MINGW__
-// rmills: dummy off for MinGW.  I don't have time to fix this for 0.24
-int main(int,const char**)
-{
-	return 0;
-}
-#else
-
 using namespace std;
 
 #ifndef  lengthof
@@ -44,11 +35,8 @@ using namespace std;
 #ifndef nil
 #define nil NULL
 #endif
-#ifndef _MAX_PATH
-#define _MAX_PATH 1024
-#endif
 
-#ifdef   _MSC_VER
+#if defined(_MSC_VER) || defined(__MINGW__)
 #include <windows.h>
 char*    realpath(const char* file,char* path);
 #define  lstat _stat
@@ -57,14 +45,21 @@ char*    realpath(const char* file,char* path);
 #define strcpy_s(d,l,s) strcpy(d,s)
 #define strcat_s(d,l,s) strcat(d,s)
 #endif
+#endif
 
-#else
+#if ! defined(_MSC_VER)
 #include <dirent.h>
 #include <unistd.h>
 #include <sys/param.h>
 #define  stricmp strcasecmp
 #endif
 
+#ifndef _MAX_PATH
+#define _MAX_PATH 1024
+#endif
+
+#define UNUSED(x) (void)(x)
+
 // prototypes
 class Options;
 int getFileType(const char* path ,Options& options);
@@ -75,12 +70,13 @@ time_t parseTime(const char* ,bool bAdjust=false);
 int    timeZoneAdjust();
 
 // platform specific code
-#ifdef   _MSC_VER
+#if defined(_MSC_VER) || defined(__MINGW__)
 char* realpath(const char* file,char* path)
 {
     char* result = (char*) malloc(_MAX_PATH);
     if   (result) GetFullPathName(file,_MAX_PATH,result,NULL);
     return result ;
+    UNUSED(path);
 }
 #endif
 
@@ -417,23 +413,25 @@ time_t parseTime(const char* arg,bool bAdjust)
 int timeZoneAdjust()
 {
     time_t    now   = time(NULL);
-    struct tm local = *localtime(&now) ;
     int       offset;
 
-#if   defined(_MSC_VER)
+#if   defined(_MSC_VER) || defined(__MINGW__)
     TIME_ZONE_INFORMATION TimeZoneInfo;
     GetTimeZoneInformation( &TimeZoneInfo );
     offset = - (((int)TimeZoneInfo.Bias + (int)TimeZoneInfo.DaylightBias) * 60);
+    UNUSED(now);
 #elif defined(__CYGWIN__)
-    struct tm lcopy  = *localtime(&now);
-    time_t    gmt    =  timegm(&lcopy) ; // timegm modifies lcopy, so don't use local
-    offset           = (int) ( ((long signed int) gmt) - ((long signed int) now) ) ;
+    struct tm lcopy = *localtime(&now);
+    time_t    gmt   =  timegm(&lcopy) ; // timegm modifies lcopy
+    offset          = (int) ( ((long signed int) gmt) - ((long signed int) now) ) ;
 #elif defined(OS_SOLARIS)
+    struct tm local = *localtime(&now) ;
     time_t local_tt = (int) mktime(&local);
     time_t time_gmt = (int) mktime(gmtime(&now));
-    offset = time_gmt - local_tt;
+    offset          = time_gmt - local_tt;
 #else
-    offset = local.tm_gmtoff ;
+    struct tm local = *localtime(&now) ;
+    offset          = local.tm_gmtoff ;
 #endif
 
 #if 0
@@ -455,7 +453,7 @@ string getExifTime(const time_t t)
 
 std::string makePath(std::string dir,std::string file)
 {
-    return dir + std::string(EXV_SEPERATOR_STR) + file ;
+    return dir + std::string(EXV_SEPARATOR_STR) + file ;
 }
 
 const char* makePath(const char* dir,const char* file)
@@ -920,12 +918,12 @@ int main(int argc,const char* argv[])
    char* filename = "crt_stat.c";
    errno_t err;
 
-   // Get data associated with "crt_stat.c": 
+   // Get data associated with "crt_stat.c":
    result = _stat( filename, &buf );
 
    int _utime(
    const char *filename,
-   struct _utimbuf *times 
+   struct _utimbuf *times
 );
    */
 
@@ -957,4 +955,6 @@ int main(int argc,const char* argv[])
 
     return result ;
 }
-#endif
+
+// That's all Folks!
+////
