@@ -1,10 +1,8 @@
 // ***************************************************************** -*- C++ -*-
-// mmap-test.cpp, $Rev: 1967 $
+// mmap-test.cpp, $Rev: 3090 $
 // Simple mmap tests
 
-#include <exiv2/basicio.hpp>
-#include <exiv2/error.hpp>
-#include <exiv2/futils.hpp>
+#include <exiv2/exiv2.hpp>
 
 #include <iostream>
 #include <cstring>
