@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004-2013 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004-2015 Andreas Huggel <ahuggel@gmx.net>
  *
  * This program is part of the Exiv2 distribution.
  *
@@ -20,30 +20,19 @@
  */
 /*
   File:    pngchunk.cpp
-  Version: $Rev: 3091 $
+  Version: $Rev: 3777 $
   Author(s): Gilles Caulier (cgilles) <caulier dot gilles at gmail dot com>
   History: 12-Jun-06, gc: submitted
   Credits: See header file
  */
 // *****************************************************************************
 #include "rcsid_int.hpp"
-EXIV2_RCSID("@(#) $Id: pngchunk.cpp 3091 2013-07-24 05:15:04Z robinwmills $")
+EXIV2_RCSID("@(#) $Id: pngchunk.cpp 3777 2015-05-02 11:55:40Z ahuggel $")
 
-// *****************************************************************************
 // included header files
-#ifdef _MSC_VER
-# include "exv_msvc.h"
-#else
-# include "exv_conf.h"
-#endif
+#include "config.h"
 
-//#define DEBUG 1
 #ifdef EXV_HAVE_LIBZ
-
-extern "C" {
-#include <zlib.h>     // To uncompress or compress text chunk
-}
-
 #include "pngchunk_int.hpp"
 #include "tiffimage.hpp"
 #include "jpgimage.hpp"
@@ -61,6 +50,8 @@ extern "C" {
 #include <cassert>
 #include <cstdio>
 
+#include <zlib.h>     // To uncompress or compress text chunk
+
 /*
 
 URLs to find informations about PNG chunks :
@@ -690,3 +681,4 @@ namespace Exiv2 {
 
 }}                                      // namespace Internal, Exiv2
 #endif // ifdef EXV_HAVE_LIBZ
+
