@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004-2010 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004-2015 Andreas Huggel <ahuggel@gmx.net>
  *
  * This program is part of the Exiv2 distribution.
  *
@@ -20,25 +20,20 @@
  */
 /*
   File:      psdimage.cpp
-  Version:   $Rev: 2370 $
+  Version:   $Rev: 3777 $
   Author(s): Marco Piovanelli, Ovolab (marco)
              Michael Ulbrich (mul)
   History:   05-Mar-2007, marco: created
  */
 // *****************************************************************************
 #include "rcsid_int.hpp"
-EXIV2_RCSID("@(#) $Id: psdimage.cpp 2370 2010-10-25 13:24:51Z ahuggel $")
+EXIV2_RCSID("@(#) $Id: psdimage.cpp 3777 2015-05-02 11:55:40Z ahuggel $")
 
-//#define DEBUG 1
-
-// *****************************************************************************
 // included header files
-#ifdef _MSC_VER
-# include "exv_msvc.h"
-#else
-# include "exv_conf.h"
-#endif
+#include "config.h"
+
 #include "psdimage.hpp"
+#include "jpgimage.hpp"
 #include "image.hpp"
 #include "basicio.hpp"
 #include "error.hpp"
@@ -56,11 +51,9 @@ EXIV2_RCSID("@(#) $Id: psdimage.cpp 2370 2010-10-25 13:24:51Z ahuggel $")
 //       Extend this helper to a proper class with all required functionality,
 //       then move it here or into a separate file?
 
-const uint32_t kPhotoshopResourceType = 0x3842494d; // '8BIM'
-
 //! @cond IGNORE
 struct PhotoshopResourceBlock {
-    uint32_t      resourceType;       // always kPhotoshopResourceType
+    uint32_t      resourceType;       // one of the markers in Photoshop::irbId_[]
     uint16_t      resourceId;
     unsigned char resourceName[2];    // Pascal string (length byte + characters), padded to an even size -- this assumes the empty string
     uint32_t      resourceDataSize;
@@ -202,7 +195,7 @@ namespace Exiv2 {
             throw Error(3, "Photoshop");
         }
 
-        // after the color data section, comes a list of resource blocks, preceeded by the total byte size
+        // after the color data section, comes a list of resource blocks, preceded by the total byte size
         if (io_->read(buf, 4) != 4)
         {
             throw Error(3, "Photoshop");
@@ -215,14 +208,11 @@ namespace Exiv2 {
                 throw Error(3, "Photoshop");
             }
 
-            // read resource type and ID
-            uint32_t resourceType = getULong(buf, bigEndian);
-            uint16_t resourceId = getUShort(buf + 4, bigEndian);
-
-            if (resourceType != kPhotoshopResourceType)
+            if (!Photoshop::isIrb(buf, 4))
             {
                 break; // bad resource type
             }
+            uint16_t resourceId = getUShort(buf + 4, bigEndian);
             uint32_t resourceNameLength = buf[6] & ~1;
 
             // skip the resource name, plus any padding
@@ -296,6 +286,54 @@ namespace Exiv2 {
                 break;
             }
 
+            // - PS 4.0 preview data is fetched from ThumbnailResource
+            // - PS >= 5.0 preview data is fetched from ThumbnailResource2
+            case kPhotoshopResourceID_ThumbnailResource:
+            case kPhotoshopResourceID_ThumbnailResource2:
+            {
+                /*
+                  Photoshop thumbnail resource header
+
+                  offset  length    name            description
+                  ======  ========  ====            ===========
+                   0      4 bytes   format          = 1 (kJpegRGB). Also supports kRawRGB (0).
+                   4      4 bytes   width           Width of thumbnail in pixels.
+                   8      4 bytes   height          Height of thumbnail in pixels.
+                  12      4 bytes   widthbytes      Padded row bytes as (width * bitspixel + 31) / 32 * 4.
+                  16      4 bytes   size            Total size as widthbytes * height * planes
+                  20      4 bytes   compressedsize  Size after compression. Used for consistentcy check.
+                  24      2 bytes   bitspixel       = 24. Bits per pixel.
+                  26      2 bytes   planes          = 1. Number of planes.
+                  28      variable  data            JFIF data in RGB format.
+                                                    Note: For resource ID 1033 the data is in BGR format.
+                */
+                byte buf[28];
+                if (io_->read(buf, 28) != 28)
+                {
+                    throw Error(3, "Photoshop");
+                }
+                NativePreview nativePreview;
+                nativePreview.position_ = io_->tell();
+                nativePreview.size_ = getLong(buf + 20, bigEndian);    // compressedsize
+                nativePreview.width_ = getLong(buf + 4, bigEndian);
+                nativePreview.height_ = getLong(buf + 8, bigEndian);
+                const uint32_t format = getLong(buf + 0, bigEndian);
+
+                if (nativePreview.size_ > 0 && nativePreview.position_ >= 0) {
+                    io_->seek(static_cast<long>(nativePreview.size_), BasicIo::cur);
+                    if (io_->error() || io_->eof()) throw Error(14);
+
+                    if (format == 1) {
+                        nativePreview.filter_ = "";
+                        nativePreview.mimeType_ = "image/jpeg";
+                        nativePreviews_.push_back(nativePreview);
+                    } else {
+                        // unsupported format of native preview
+                    }
+                }
+                break;
+            }
+
             default:
             {
                 break;
@@ -399,8 +437,9 @@ namespace Exiv2 {
             // read resource type and ID
             uint32_t resourceType = getULong(buf, bigEndian);
 
-            if (resourceType != kPhotoshopResourceType) {
-                break; // bad resource type
+            if (!Photoshop::isIrb(buf, 4))
+            {
+                throw Error(3, "Photoshop"); // bad resource type
             }
             uint16_t resourceId = getUShort(buf + 4, bigEndian);
             uint32_t resourceNameLength = buf[6];
@@ -416,13 +455,13 @@ namespace Exiv2 {
             if (io_->read(buf, 4) != 4) throw Error(3, "Photoshop");
 
             uint32_t resourceSize = getULong(buf, bigEndian);
+            uint32_t pResourceSize = (resourceSize + 1) & ~1;    // padded resource size
             uint32_t curOffset = io_->tell();
 
             // Write IPTC_NAA resource block
             if ((resourceId == kPhotoshopResourceID_IPTC_NAA  ||
                  resourceId >  kPhotoshopResourceID_IPTC_NAA) && iptcDone == false) {
                 newResLength += writeIptcData(iptcData_, outIo);
-                resourceSize = (resourceSize + 1) & ~1;    // adjust for padding
                 iptcDone = true;
             }
 
@@ -430,7 +469,6 @@ namespace Exiv2 {
             else if ((resourceId == kPhotoshopResourceID_ExifInfo  ||
                       resourceId >  kPhotoshopResourceID_ExifInfo) && exifDone == false) {
                 newResLength += writeExifData(exifData_, outIo);
-                resourceSize = (resourceSize + 1) & ~1;    // adjust for padding
                 exifDone = true;
             }
 
@@ -438,7 +476,6 @@ namespace Exiv2 {
             else if ((resourceId == kPhotoshopResourceID_XMPPacket  ||
                       resourceId >  kPhotoshopResourceID_XMPPacket) && xmpDone == false) {
                 newResLength += writeXmpData(xmpData_, outIo);
-                resourceSize = (resourceSize + 1) & ~1;    // adjust for padding
                 xmpDone = true;
             }
 
@@ -447,11 +484,12 @@ namespace Exiv2 {
                 && resourceId != kPhotoshopResourceID_ExifInfo
                 && resourceId != kPhotoshopResourceID_XMPPacket) {
 #ifdef DEBUG
+                std::cerr << std::hex << "copy : resourceType: " << resourceType << "\n";
                 std::cerr << std::hex << "copy : resourceId: " << resourceId << "\n";
                 std::cerr << std::dec;
 #endif
                 // Copy resource block to new PSD file
-                ul2Data(buf, kPhotoshopResourceType, bigEndian);
+                ul2Data(buf, resourceType, bigEndian);
                 if (outIo.write(buf, 4) != 4) throw Error(21);
                 us2Data(buf, resourceId, bigEndian);
                 if (outIo.write(buf, 2) != 2) throw Error(21);
@@ -467,10 +505,9 @@ namespace Exiv2 {
 
                 readTotal = 0;
                 toRead = 0;
-                resourceSize = (resourceSize + 1) & ~1;        // pad to even
-                while (readTotal < resourceSize) {
-                    toRead =   static_cast<long>(resourceSize - readTotal) < lbuf.size_
-                             ? static_cast<long>(resourceSize - readTotal) : lbuf.size_;
+                while (readTotal < pResourceSize) {
+                    toRead =   static_cast<long>(pResourceSize - readTotal) < lbuf.size_
+                             ? static_cast<long>(pResourceSize - readTotal) : lbuf.size_;
                     if (io_->read(lbuf.pData_, toRead) != toRead) {
                         throw Error(3, "Photoshop");
                     }
@@ -478,11 +515,11 @@ namespace Exiv2 {
                     if (outIo.write(lbuf.pData_, toRead) != toRead) throw Error(21);
                 }
                 if (outIo.error()) throw Error(21);
-                newResLength += resourceSize + adjResourceNameLen + 12;
+                newResLength += pResourceSize + adjResourceNameLen + 12;
             }
 
-            io_->seek(curOffset + resourceSize, BasicIo::beg);
-            oldResLength -= (12 + adjResourceNameLen + resourceSize);
+            io_->seek(curOffset + pResourceSize, BasicIo::beg);
+            oldResLength -= (12 + adjResourceNameLen + pResourceSize);
         }
 
         // Append IPTC_NAA resource block, if not yet written
@@ -503,6 +540,10 @@ namespace Exiv2 {
             xmpDone = true;
         }
 
+        // Populate the fake data, only make sense for remoteio, httpio and sshio.
+        // it avoids allocating memory for parts of the file that contain image-date.
+        io_->populateFakeData();
+
         // Copy remaining data
         long readSize = 0;
         while ((readSize=io_->read(lbuf.pData_, lbuf.size_))) {
@@ -532,8 +573,7 @@ namespace Exiv2 {
                 std::cerr << std::hex << "write: resourceId: " << kPhotoshopResourceID_IPTC_NAA << "\n";
                 std::cerr << std::dec << "Writing IPTC_NAA: size: " << rawIptc.size_ << "\n";
 #endif
-                ul2Data(buf, kPhotoshopResourceType, bigEndian);
-                if (out.write(buf, 4) != 4) throw Error(21);
+                if (out.write(reinterpret_cast<const byte*>(Photoshop::irbId_[0]), 4) != 4) throw Error(21);
                 us2Data(buf, kPhotoshopResourceID_IPTC_NAA, bigEndian);
                 if (out.write(buf, 2) != 2) throw Error(21);
                 us2Data(buf, 0, bigEndian);                      // NULL resource name
@@ -573,8 +613,7 @@ namespace Exiv2 {
                 std::cerr << std::hex << "write: resourceId: " << kPhotoshopResourceID_ExifInfo << "\n";
                 std::cerr << std::dec << "Writing ExifInfo: size: " << blob.size() << "\n";
 #endif
-                ul2Data(buf, kPhotoshopResourceType, bigEndian);
-                if (out.write(buf, 4) != 4) throw Error(21);
+                if (out.write(reinterpret_cast<const byte*>(Photoshop::irbId_[0]), 4) != 4) throw Error(21);
                 us2Data(buf, kPhotoshopResourceID_ExifInfo, bigEndian);
                 if (out.write(buf, 2) != 2) throw Error(21);
                 us2Data(buf, 0, bigEndian);                      // NULL resource name
@@ -618,8 +657,7 @@ namespace Exiv2 {
             std::cerr << std::hex << "write: resourceId: " << kPhotoshopResourceID_XMPPacket << "\n";
             std::cerr << std::dec << "Writing XMPPacket: size: " << xmpPacket.size() << "\n";
 #endif
-            ul2Data(buf, kPhotoshopResourceType, bigEndian);
-            if (out.write(buf, 4) != 4) throw Error(21);
+            if (out.write(reinterpret_cast<const byte*>(Photoshop::irbId_[0]), 4) != 4) throw Error(21);
             us2Data(buf, kPhotoshopResourceID_XMPPacket, bigEndian);
             if (out.write(buf, 2) != 2) throw Error(21);
             us2Data(buf, 0, bigEndian);                      // NULL resource name
