@@ -26,11 +26,13 @@
 using namespace std;
 
 #if XMP_WinBuild
+#ifdef _MSC_VER
 	#pragma warning ( disable : 4533 )	// initialization of '...' is skipped by 'goto ...'
 	#pragma warning ( disable : 4702 )	// unreachable code
 	#pragma warning ( disable : 4800 )	// forcing value to bool 'true' or 'false' (performance warning)
 	#pragma warning ( disable : 4996 )	// '...' was declared deprecated
 #endif
+#endif
 
 
 // *** Use the XMP_PropIsXyz (Schema, Simple, Struct, Array, ...) macros
@@ -1165,7 +1167,7 @@ XMPMeta::ParseFromBuffer ( XMP_StringPtr  buffer,
 			
 				// We've got some leftover input, process it first then continue with the current
 				// buffer. Try to fill the pendingInput buffer before parsing further. We use a loop
-				// for wierd edge cases like a 2 byte input buffer, using 1 byte for pendingInput,
+				// for weird edge cases like a 2 byte input buffer, using 1 byte for pendingInput,
 				// then having a partial UTF-8 end and need to absorb more.
 				
 				size_t pendingOverlap = kXMLPendingInputMax - parser.pendingCount;
